% Matlab code to estimate a mixed logit model with hierachical Bayes procedures
% Written by Kenneth Train, first version July 13, 2006, 
%   latest edits July 27, 2006

% Modified for the needs of:
% Czajkowski, M., Giergiczny, M., Kronenberg, J., and Englin, J. E., The Individual Travel Cost Method with Consumer-Specific Values of Travel Time Savings.


%% DATA and MODEL SPECIFICATION

clear all

% Declare GLOBAL variables
global NP NCS NROWS
global IDV NV NAMES FULLCV
global A B C D  
global RHO NCREP NEREP NSKIP NRFOR NRLL INFOSKIP
global SEED1 SEED2
global KEEPMN PUTA PUTD WANTINDC PUTC
global NALTMAX NCSMAX
global X S A B C D SQRTNP 
global XF F NF IDF NAMESF RHOF
global XMAT
global cmeans2 sdev2

DATA = load('Dataset_dce.mat');

XMAT = [DATA.resno, DATA.CS, DATA.ANS, DATA.alt==1, DATA.time./60, -DATA.fee/1];

% identify choice tasks with missing responses
ans_missing = repmat(sum(reshape(DATA.ANS,[3,size(XMAT,1)/3]),1) ~= 1,[3,1]);
ans_missing = ans_missing(:);

% restructure data for Train's code input format
resno_tmp = DATA.resno;
resno_tmp((DATA.Q16b == 1) | (ans_missing == 1) | (DATA.ZYW ~= 1),:) = [];
resno_tmp = unique(resno_tmp);
resno_tmp(:,2) = 1:size(resno_tmp,1);

resno_tmp2 = DATA.resno;
resno_tmp2((ans_missing == 1) | (DATA.ZYW ~= 1),:) = [];
resno_tmp2 = unique(resno_tmp2);

resno_tmp3 = DATA.resno;
resno_tmp3((DATA.ZYW ~= 1) | (DATA.Q16b == 1),:) = [];
resno_tmp3 = unique(resno_tmp3); % ZYW and WTP>0

resno_tmp4 = unique(~ismember(resno_tmp3,resno_tmp(:,1)) .* resno_tmp3); % WTP>0 but no choices
resno_tmp4(1) = [];

resno_tmp5 = DATA.resno;
resno_tmp5((DATA.ZYW ~= 1) | (DATA.Q16b ~= 1) ,:) = [];
resno_tmp5 = unique(resno_tmp5); % ZYW and WTP = 0

resno_tmp6 = DATA.resno;
resno_tmp6((DATA.ZYW ~= 1),:) = [];
resno_tmp6 = unique(resno_tmp6); % all ZYW


XMAT((DATA.Q16b == 1) | (ans_missing == 1) | (DATA.ZYW ~= 1),:) = []; % 0 choice tasks completed or did not wish their trip was shorter
XMAT(:,1) = vlookup(XMAT(:,1),resno_tmp(:,1),resno_tmp(:,2));

% resno_tmp(:,3) = histc(XMAT(:,1),resno_tmp(:,2))/3;
% alt_tmp = [];
% for i=1:size(resno_tmp,1)
%     alt_tmp = [alt_tmp, 1:resno_tmp(i,3)];
% end
% alt_tmp = repmat(alt_tmp,[3,1]);
% alt_tmp = alt_tmp(:);
% XMAT(:,2) = alt_tmp;
XMAT(:,2) = reshape(repmat(1:size(XMAT,1)/3,[3,1]),[size(XMAT,1),1]);

% The variables in XMAT are:
% 1. Person number (1-NP)            MUST BE THIS. DO NOT CHANGE.
% 2. Choice situation number (1-NCS) MUST BE THIS. DO NOT CHANGE.
% 3. Chosen alternative (1/0)        MUST BE THIS. DO NOT CHANGE.
% 4. SQ 
% 5. time savings (h)
% 6. fee (100 PLN)
NAMES = {'SQ';'time';'fee'}; 

% Number of people (decision-makers) in dataset 
NP=XMAT(end,1); %292; % a - 590, z - 501, w - 312

% Number of choice situations in dataset. This is the number faced by all the people combined.
NCS=length(XMAT)/3;

% Total number of alternatives faced by all people in all choice situations combined.
% This is the number of rows of data in XMAT below.
NROWS=NCS*3;


% MODEL SPECIFICATION

% RANDOM COEFFICIENTS
IDV = [];

% IDV = [ 4 1; % normal
%         5 3; % truncated normal (with the share below zero massed at zero)
%         6 3]; % truncated normal (with the share below zero massed at zero)

NV = size(IDV,1); %Number of random coefficients. 

% Starting values
A = [];
% A = [-0.9;0.8;12.6];
B = repmat(A,1,NP);    %Start each person at the starting means
D = NV.*eye(NV);       %Starting variances equal to number of random coefficients

% Set the initial proportionality fraction for the jumping distribution for the random coefficients for each person.
% This fraction is adjusted by the program in each iteration to attain an acceptance rate of about .3 in the M-H algorithm for the B's 
RHO = 0.3;

% FIXED COEFFICIENTS
% IDF = [];
IDF = [4; 5; 6];

NF = size(IDF,1); 

% Give a name to each of the variables in IDF.
% NAMESF = {''};
NAMESF = {'SQ';'time';'fee'}; 

% Starting values.
% F = []; 
F = [0.59;0.16;0.75];
% F = [0.059;0.16;0.75];
F = [0.593012693137937;0.164941667280156;0.00767201670784903]

% Set the proportionality fraction for the jumping distribution for the fixed coefficients.
% This fraction is not adjusted by the program. Ideally, RHOF should be set such the acceptance rate is about .3 in the M-H algorithm for the draws of F 
RHOF = 0.00007;

% BURN-IN, RETAINING DRAWS, PRINTING ITERATION INFORMATION

% Number of iterations to make prior to retaining draws (i.e., length of burn-in)
NCREP = 1e5; % 10000

% Number of draws to retain after burn-in 
NEREP = 1e5; % 10000

% Number of iterations to make between retained draws
NSKIP = 11; % 10

% Number of draws from N(A-hat,D-hat) to use in simulating the estimated dist of random coefficients.
NRFOR = 1e6; % 2000

% Number of draws per person to use in calculating the simulated log-likelihood value at A-hat, D-hat, and F-hat.
NRLL = 1e6; % 2000

% How frequently to print information about the iteration process eg 100 == print out info every 100-th iteration 
INFOSKIP = 1e3;

% Set seeds for the random number generators.
% Seed for random draws in iteration process. Must be a positive integer.
SEED1 = 179424673;   
% Seed for random draws fron N(A-hat,D-hat) in simulated distribution of
% coefficients and log-likelihood value. Must be a positive integer.
SEED2 = 13375731;   

% Do you want to save the draws of A, D, and F to a file?  If so, set KEEPMN=1. If no, set KEEPMN=0.
KEEPMN = 1;

% If KEEPMN=1, specify filenames to which to save the draws of A, D, and F.
PUTA = 'drawsA';
PUTD = 'drawsD';
PUTF = 'drawsF';

% Do you want to calculate and save the means of the NEREP draws of the individual-level random coefficients? If so, set WANTINDC=1. If no, set WANTINDC=0.
WANTINDC=1;

% If WANTINDC=1, specify filename (and full path if not the working directory) to which to save the mean of the draws of the individual-level random coefficients. 
% The output in FOUTPUTC will be a matlab matrix of dimension NPxNIV, containing one row for each  person and one column for each coefficient. 
PUTC = 'meansC';

doit

%% MNL estimates:

mnl_results = [fhat, fsd, pv(fhat, fsd)]
mod_diag_mnl = mod_diag;

[~, ~, raw0_0] = xlsread('C:\Documents\MATLAB\bociany\Dataset_tcm.xls','Arkusz1','A2:A701');
[~, ~, raw0_1] = xlsread('C:\Documents\MATLAB\bociany\Dataset_tcm.xls','Arkusz1','DK2:DM701');
raw = [raw0_0,raw0_1];
raw(cellfun(@(x) ~isempty(x) && isnumeric(x) && isnan(x),raw)) = {''};
R = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw); % Find non-numeric cells
raw(R) = {NaN}; % Replace non-numeric cells
data = reshape([raw{:}],size(raw));
resno = data(:,1);
time = data(:,2);
HHinc = data(:,3);
HHinc_person = data(:,4);
clearvars data raw raw0_0 raw0_1 R;
wage = HHinc_person / 40 / 4;

% VTTS MNL:
VTTS_mnl = mean(1./mnl_results(3,1) .* 1 .* (...
    log(exp(mnl_results(1,1) + mnl_results(2,1).*time) + exp(mnl_results(2,1).*(time))) - ...
    log(exp(mnl_results(1,1) + mnl_results(2,1).*time) + exp(mnl_results(2,1).*(time-1)))))
% VTTS_mnl2 = mean(1./mnl_results(3,1) .* 10 .* (...
%     log(exp(mnl_results(1,1) + mnl_results(2,1).*time) + exp(mnl_results(2,1).*(time-1))) - ...
%     log(exp(mnl_results(1,1) + mnl_results(2,1).*time)                                  ) ))
% VTTS_mnl3 = mnl_results(2,1)./mnl_results(3,1) .* 10

% 95% c.i. for VTTS MNL
% sim1 = 1e4;
% rng(SEED1)
% B_draws = mvnrnd(mnl_results(:,1),diag(mnl_results(:,2).^2),sim1);
% VTTS_mnl_i = 1./B_draws(:,3) .* 1 .* (...
%     log(exp(B_draws(:,1) + B_draws(:,2).*2 ) + exp(B_draws(:,2).*2)) - ...
%     log(exp(B_draws(:,1) + B_draws(:,2).*2 ) + exp(B_draws(:,2).*(2-1))) );
% 
% VTTS_MNL_ci = [VTTS_mnl(1) quantile(VTTS_mnl_i,0.025),quantile(VTTS_mnl_i,0.975)] 

VTTS = NaN(size(resno,1),3);
VTTS(:,1) = resno;
VTTS(resno_tmp(:,1),2) = VTTS_mnl;


%% MXL

% MODEL SPECIFICATION

% XMAT(:,6) = XMAT(:,6)*10;

% Correlation
FULLCV = 0; % estimate covariance among all random coefficients

% RANDOM COEFFICIENTS
IDV = [ 4 1; % normal
        5 2; % lognormal
        6 2]; % lognormal

NV = size(IDV,1); %Number of random coefficients. 

% Starting values
% A = [-1.1;0.8;1.3];
% A = [-1.50;-0.98;1.99];
A = [-1.89918963369120;-0.0430679700899094;-2.02665123493737];
B = repmat(A,1,NP);    %Start each person at the starting means
D = NV.*eye(NV);       %Starting variances equal to number of random coefficients
% D = [7.27100783893493,0,0;...
%     -0.817692106155370,-2.74917676525964,0;...
%     0.311884214668343,-2.25628641332899,0.375551551141018];
% D = D*D';
% RHO = 0.03;

% FIXED COEFFICIENTS
IDF = [];

NF = size(IDF,1); 

% Give a name to each of the variables in IDF.
NAMESF = {''};

% Starting values.
F = []; 

doit



%% MXL estimates:

mxl_results = [ahat, asd, pv(ahat, asd), sqrt(diag(dhat)), diag(dsd)./2./sqrt(diag(dhat)),pv(sqrt(diag(dhat)), diag(dsd)./2./sqrt(diag(dhat)))]
mod_diag_mxl = mod_diag;

% calculations first, then median:

cmeans2_tmp2 = cmeans2;
cmeans2_tmp2(:,2:3,:) = exp(cmeans2_tmp2(:,2:3,:));

VTTS_mxl_i = squeeze((1./cmeans2_tmp2(:,3,:) .* 1 .* (...
    log(exp(cmeans2_tmp2(:,1,:) + cmeans2_tmp2(:,2,:).*time(resno_tmp(:,1))) + exp(cmeans2_tmp2(:,2,:).*time(resno_tmp(:,1)))) - ...
    log(exp(cmeans2_tmp2(:,1,:) + cmeans2_tmp2(:,2,:).*time(resno_tmp(:,1))) + exp(cmeans2_tmp2(:,2,:).*(time(resno_tmp(:,1))-1))) )));

VTTS_mxl = nanmedian(VTTS_mxl_i,2);

% VTTS_mxl1 = (nanmedian(1./cmeans2_tmp2(:,3,:) .* 10 .* (...
%         log(exp(cmeans2_tmp2(:,1,:) + cmeans2_tmp2(:,2,:).*time(resno_tmp(:,1))) + exp(cmeans2_tmp2(:,2,:).*(time(resno_tmp(:,1))-1))) - ...
%     log(exp(cmeans2_tmp2(:,1,:) + cmeans2_tmp2(:,2,:).*time(resno_tmp(:,1))) ) ),3));
% VTTS_mxl2 = (nanmedian(cmeans2_tmp2(:,2,:)./cmeans2_tmp2(:,3,:),3).*10);
% 
% mean([VTTS_mxl, VTTS_mxl1, VTTS_mxl2])
% 
% % median first, then calculations :
% 
% cmeans2_tmp1 = median(cmeans2,3);
% 
% VTTS_mxl = ((1./cmeans2_tmp1(:,3) .* 10 .* (...
%     log(exp(cmeans2_tmp1(:,1) + cmeans2_tmp1(:,2).*time(resno_tmp(:,1))) + exp(cmeans2_tmp1(:,2).*time(resno_tmp(:,1)))) - ...
%     log(exp(cmeans2_tmp1(:,1) + cmeans2_tmp1(:,2).*time(resno_tmp(:,1))) + exp(cmeans2_tmp1(:,2).*(time(resno_tmp(:,1))-1))) )));
% 
% VTTS_mxl1 = ((1./cmeans2_tmp1(:,3) .* 10 .* (...
%         log(exp(cmeans2_tmp1(:,1) + cmeans2_tmp1(:,2).*time(resno_tmp(:,1))) + exp(cmeans2_tmp1(:,2).*(time(resno_tmp(:,1))-1))) - ...
%     log(exp(cmeans2_tmp1(:,1) + cmeans2_tmp1(:,2).*time(resno_tmp(:,1))) ) )));
% 
% VTTS_mxl2 = ((cmeans2_tmp1(:,2)./cmeans2_tmp1(:,3)).*10);
% 
% mean([VTTS_mxl, VTTS_mxl1, VTTS_mxl2])

VTTS(resno_tmp(:,1),3) = vlookup(resno_tmp(:,1),resno_tmp(:,1),VTTS_mxl);


% % simulate using model estimates:
% sim2 = 1e4;
% SEED1 = 179424673;   
% rng(SEED1)
% B_draws = mvnrnd(ahat,dhat,sim2);
% B_draws(:,2:3) = exp(B_draws(:,2:3));
% VTTS_mxl_i = 1./B_draws(:,3) .* 10 .* (...
%     log(exp(B_draws(:,1) + B_draws(:,2).*mean(time) ) + exp(B_draws(:,2).*(mean(time)))) - ...
%     log(exp(B_draws(:,1) + B_draws(:,2).*mean(time) ) + exp(B_draws(:,2).*(mean(time)-1))) );
% VTTS_mxl_i2 = B_draws(:,2)./B_draws(:,3) .* 10;
% [nanmean(VTTS_mxl_i) nanmedian(VTTS_mxl_i)]
% [mean(VTTS_mxl_i2) median(VTTS_mxl_i2)]

%% table 2:

% resno_tmp6 - 581 total ZYW
% resno_tmp5 - 273 nie chc? p?aci?
% resno_tmp3 - 308 chc? p?aci?
% resno_tmp4 - 50 chc? p?aci? ale missing
% resno_tmp(:,1) - 258 w modelu


VTTS = [VTTS,wage];
corr(VTTS(~isnan(VTTS(:,3)) & ~isnan(VTTS(:,4)),3),VTTS(~isnan(VTTS(:,3)) & ~isnan(VTTS(:,4)),4)) % only those who WTP and non-missing
% VTTS_tmp = VTTS;
VTTS_mean_yes = [nanmean(VTTS(:,2)), nanmean(VTTS(:,3))];
VTTS(resno_tmp4,2) = VTTS_mean_yes(1); % stated WTP > 0 but made no choices - assume mean WTP of the others who WTP>0
VTTS(resno_tmp4,3) = VTTS_mean_yes(2); % stated WTP > 0 but made no choices - assume mean WTP of the others who WTP>0
VTTS(resno_tmp5,2:3) = 0; % stated WTP = 0
corr(VTTS(~isnan(VTTS(:,3)) & ~isnan(VTTS(:,4)),3),VTTS(~isnan(VTTS(:,3)) & ~isnan(VTTS(:,4)),4)) % all ZYW


T2 = [[mean(VTTS(resno_tmp6,3));median(VTTS(resno_tmp6,3));std(VTTS(resno_tmp6,3));quantile(VTTS(resno_tmp6,3),0.025);quantile(VTTS(resno_tmp6,3),0.975)],...
     [mean(VTTS(resno_tmp3,3));median(VTTS(resno_tmp3,3));std(VTTS(resno_tmp3,3));quantile(VTTS(resno_tmp3,3),0.025);quantile(VTTS(resno_tmp3,3),0.975)],...
     [nanmean(wage(resno_tmp6,:));nanmedian(wage(resno_tmp6,:));nanstd(wage(resno_tmp6,:));quantile(wage(resno_tmp6,:),0.025);quantile(wage(resno_tmp6,:),0.975)],...
     [nanmean(wage(resno_tmp3,:));nanmedian(wage(resno_tmp3,:));nanstd(wage(resno_tmp3,:));quantile(wage(resno_tmp3,:),0.025);quantile(wage(resno_tmp3,:),0.975)]]

VTTS_z = VTTS(resno_tmp6,:); % this is used for the bubble plot and TCM

VTTS_mxl_i = vlookup(resno, resno_tmp(:,1), VTTS_mxl_i);
VTTS_mxl_i(resno_tmp4,:) = VTTS_mean_yes(2);
VTTS_mxl_i = VTTS_mxl_i(resno_tmp6,:);

clear cmeans2_tmp
save mxl_bayesian_out