% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%           Count Data Model - Negative-Binomial type 2 + truncation and endogenous strat
% 
%           Credits, acknowledgements, references: 
%           - Klaus Moeltner's Bayesian Econometric Analysis course (http://faculty.agecon.vt.edu/moeltner/AAEC6984.html)
%           - Greene, W. H., 2011. Econometric Analysis. 7 Ed., Prentice Hall, Upper Saddle River, NJ.
%           - Greene, W., 2008. Functional forms for the negative binomial model for count data. Economics Letters, 99(3):585-590.
%           - Greene, W. H., 2013. NLOGIT Reference Guide. Econometric Software, Plainview, NY.
%           - Chib, S., and Greenberg, E., 1995. Understanding the Metropolis-Hastings Algorithm. The American Statistician, 49(4):327-335.
%           - Chib, S., Greenberg, E., and Winkelmann, R., 1998. Posterior simulation and Bayes factors in panel count data models. Journal of Econometrics, 86(1):33-54.
%           
%           TCM - stork village data 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
clc

tic;  % start stop watch

% *****************  loading & specifying data ****************************

DATA = load('dataset_tcm.mat');

% This is for draw-specific data, NaNs and extreme values make this approach infeasible
% load('mxl_bayesian_out','VTTS_mxl_i');
% VTTS_mxl_i = min(VTTS_mxl_i,repmat(quantile(VTTS_mxl_i,0.975,2),[1,size(VTTS_mxl_i,2)])); % truncate extreme VTTS
% for i=1:size(VTTS_mxl_i,1)
%     VTTS_mxl_i(i,isnan(VTTS_mxl_i(i,:))) = quantile(VTTS_mxl_i(i,:),0.975); 
% end

inputnames = fieldnames(DATA);
idx = DATA.ZYW == 1;
for i=1:length(inputnames)
    tmp = DATA.(inputnames{(i)});
    DATA.(inputnames{(i)}) = tmp(idx);    
end
clear idx inputnames tmp i

EstimOpt.NP = 581; % no. of people
EstimOpt.NS = 1; % no. of sites
EstimOpt.NVARf = 2; % no. of fixed variables

EstimOpt.NB = 3; % 3 - NB with truncation and endogenous stratification (Englin and Shonkwiler, 2005)

% **************  estimation and optimization options  ********************

rng(179424673); % set arbitrary seed for pseudo-random draws

EstimOpt.r1 = 9e4; % burn-ins
EstimOpt.r2 = 1e4; % keepers
EstimOpt.nskip = 11; % no of draws to skip after burn-in (after burn-in there are r2*nskip iterations)

EstimOpt.MH = 1; % 0 for Gibbs sampler, 1 for independence chain MH

EstimOpt.disp = 1e4;

% Gibbs sampler:
EstimOpt.tauc = 2; %tuner for variance in MVN distribution

% fixed coefficients:
EstimOpt.tauf = 2; %tuner for variance in t-distribution
EstimOpt.vf = 8; %degrees of freedom in t-distribution
EstimOpt.MHalgf = 'tailored mvt';

EstimOpt.options = optimoptions('fminunc');
EstimOpt.options = optimoptions(EstimOpt.options,'display','off','TolFun',1e-4,'TolX',1e-4,'GradObj','on','Algorithm','quasi-newton');

if EstimOpt.NB == 0 
    EstimOpt.options = optimoptions(EstimOpt.options,'Hessian','on');
else
    EstimOpt.options = optimoptions (EstimOpt.options,'Hessian','off');
end

format shortG;
format compact;

% *****************************  househeeping  ****************************

EstimOpt.NOBS = EstimOpt.NP * EstimOpt.NS;
EstimOpt.NVAR = EstimOpt.NVARf;
EstimOpt.NPAR = EstimOpt.NB;
if EstimOpt.NB == 3; EstimOpt.NPAR = EstimOpt.NPAR - 2; end

y = DATA.TRIPS; % no. of trips
y(y>20) = 20;

DATA.wage = DATA.HHinc_person./160;
DATA.wage(isnan(DATA.wage)) = nanmean(DATA.wage); % this is somewhat arbitrary 


X = [ones(EstimOpt.NOBS,1) DATA.tc_km_old*2/1000];
TCstat(1,:) = [mean(X(:,2)) median(X(:,2)) std(X(:,2)) quantile(X(:,2),0.025) quantile(X(:,2),0.975)]*1000/2;
% X = [ones(EstimOpt.NOBS,1) (DATA.tc_km_old + (1-DATA.Q16b).*DATA.time.*DATA.wage/3)*2/1000];
% TCstat(2,:) = [mean(X(:,2)) median(X(:,2)) std(X(:,2)) quantile(X(:,2),0.025) quantile(X(:,2),0.975)]*1000/2;
% X = [ones(EstimOpt.NOBS,1) (DATA.tc_km_old + (1-DATA.Q16b).*DATA.time.*DATA.wage)*2/1000];
% TCstat(3,:) = [mean(X(:,2)) median(X(:,2)) std(X(:,2)) quantile(X(:,2),0.025) quantile(X(:,2),0.975)]*1000/2;
% X = [ones(EstimOpt.NOBS,1) (DATA.tc_km_old + DATA.time.*DATA.VTTS_mnl)*2/1000];
% TCstat(4,:) = [mean(X(:,2)) median(X(:,2)) std(X(:,2)) quantile(X(:,2),0.025) quantile(X(:,2),0.975)]*1000/2;
% X = [ones(EstimOpt.NOBS,1) (DATA.tc_km_old + DATA.time.*DATA.VTTS_mxl)*2/1000];
% TCstat(5,:) = [mean(X(:,2)) median(X(:,2)) std(X(:,2)) quantile(X(:,2),0.025) quantile(X(:,2),0.975)]*1000/2;

% draw specific individual TC
% X = [ones(EstimOpt.NOBS,1,size(VTTS_mxl_i,2)) , reshape((DATA.tc_km_old + DATA.time.*VTTS_mxl_i)*2/1000,[EstimOpt.NOBS,1,size(VTTS_mxl_i,2)])];
% TCstat(6,:) = [mean(X(:,2)) median(X(:,2)) std(X(:,2)) quantile(X(:,2),0.025) quantile(X(:,2),0.975)]*1000/2;

% TCstat = TCstat'

if size(X,1) ~= EstimOpt.NOBS 
    error('Number of observations / people / sites not consistent')
end

if size(X,2) ~= EstimOpt.NVAR 
    error('Number of total / fixed / random variables not consistent')
end

if EstimOpt.MH == 0
    EstimOpt.R = EstimOpt.r1 + EstimOpt.r2*EstimOpt.nskip;
else
    EstimOpt.R = EstimOpt.r1 + EstimOpt.r2;
end

% starting values:
% EstimVar.bols = (X'*X) \ X'*y;
Xtmp = median(X,3);
EstimVar.bols = (Xtmp'*Xtmp) \ Xtmp'*y;

EstimVar.the0 = 2;
EstimVar.P0 = 2;

% fixed coefficients
EstimVar.b0f = zeros(EstimOpt.NVARf + EstimOpt.NPAR,1);
EstimVar.vb0f = eye(EstimOpt.NVARf + EstimOpt.NPAR)*1e1;
EstimVar.betadraw = [EstimVar.bols(1:EstimOpt.NVARf)];
if EstimOpt.NB > 0
    EstimVar.betadraw = [EstimVar.betadraw; log(EstimVar.the0)];
    if EstimOpt.NB == 2
        EstimVar.betadraw = [EstimVar.betadraw; EstimVar.P0];
    end
end
EstimVar.betadraw = [-5.49664742590420;-2.56190354605034;4.54057442976426];


% EstimVar.Vc = eye(EstimOpt.NVARf+EstimOpt.NPAR);
% EstimVar.Vc = cov(Results.betamat')
EstimVar.Vc = [1.04130178584082,-0.0419012179044816,-1.04593243147440;-0.0419012179044816,0.845859501860456,-0.000808459860607647;-1.04593243147440,-0.000808459860607647,1.05996604897471];

% Open log file 
EstimOpt.fid = fopen('stork_nb03.txt','w');

Results = HDCM(y,X,EstimOpt,EstimVar);
 
% [betamat,gmat,gimat,Emat,acceptmat]=gs_HP(y,X,k1,k2,k,N,J,n,...
%     mub0,Vb0,taub,vb,betadraw,...
%     mug0,Vg0,taug,vg,v0,S0,Edraw,...
%     r1,r2, options);

fprintf('GS done');

% simulate using model estimates:
sim2 = 1e6;
SEED1 = 179424673;   
rng(SEED1)
CS_i = -1./normrnd(Results.kdiag(2,1),Results.kdiag(2,2),[sim2,1])*1e3;
CSsim = [median(CS_i); quantile(CS_i,0.025); quantile(CS_i,0.975)];
Results.R(5:7,1) = CSsim;

EstimOpt.tocnote = toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Capture key results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(EstimOpt.fid,'total number of iterations =\t%6.0f \n',EstimOpt.R);
fprintf(EstimOpt.fid,'burn-in iterations =\t%6.0f \n',EstimOpt.r1);
fprintf(EstimOpt.fid,'tuner for mvt variance in MH part, betaf=\t%6.3f \n',EstimOpt.tauf);
fprintf(EstimOpt.fid,'DoF in mvt=\t%6.3f \n',EstimOpt.vf);
fprintf(EstimOpt.fid,'tuner for mvt variance in MH part, betar=\t%6.3f \n',EstimOpt.tauf);
fprintf(EstimOpt.fid,'DoF in mvt=\t%6.3f \n',EstimOpt.vf);
fprintf(EstimOpt.fid,'acceptance rates in MH part for betaf =\t%6.3f \n',Results.acceptmat(1));
fprintf(EstimOpt.fid,'acceptance rates in MH part for betar =\t%6.3f \n',Results.acceptmat(2));
fprintf(EstimOpt.fid,'\n');

% beta stuff
out = Results.kdiag(1:EstimOpt.NVARf,:)';
fprintf(EstimOpt.fid,'Output table for betas \n\n');
fprintf(EstimOpt.fid,'mean\t\tstd\t\tp(>0)\t\tnse\t\tIEF\t\tm*\t\tcd\n\n');
fprintf(EstimOpt.fid,'%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\n',out);
fprintf(EstimOpt.fid,'\n');

% gamma stuff
out = Results.kdiag(EstimOpt.NVARf+1:EstimOpt.NVAR,:)';
fprintf(EstimOpt.fid,'Output table for betas \n\n');
fprintf(EstimOpt.fid,'mean\t\tstd\t\tp(>0)\t\tnse\t\tIEF\t\tm*\t\tcd\n\n');
fprintf(EstimOpt.fid,'%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\n',out);
fprintf(EstimOpt.fid,'\n');

% E stuff
out = Results.kdiag(EstimOpt.NVAR+1:end,:)';
fprintf(EstimOpt.fid,'Output table for E \n\n');
fprintf(EstimOpt.fid,'mean\t\tstd\t\tp(>0)\t\tnse\t\tIEF\t\tm*\t\tcd\n\n');
fprintf(EstimOpt.fid,'%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\t%6.3f\n',out);
fprintf(EstimOpt.fid,'\n');

disp(['Estimation took ' num2str(EstimOpt.tocnote) ' seconds ('  num2str(floor(EstimOpt.tocnote/(60*60))) ' hours ' num2str(floor(rem(EstimOpt.tocnote,60*60)/60)) ' minutes ' num2str(rem(EstimOpt.tocnote,60)) ' seconds).']);


st = fclose(EstimOpt.fid);
if st == 0
   disp('File closed successfully');
else
   warning('Problem with closing file');
end;

clear out st

type stork_nb03.txt
disp(Results.R)

% save stork_results Results;
